#    Copyright 2008, 2009 Shelby M. Tucker
#        Email:     shelby.tucker@gmail.com
#
#    This file is part of Teletory Maker.
#
#    Teletory Maker is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Teletory Maker is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Teletory Maker.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement

import httplib
import re
import string
import sys, os
import urllib
import thread
import time
from PyQt4.QtGui import *
from PyQt4 import QtCore
from pdfoutput import *

#Import the classes for our GUI
from teletorygui import Ui_MainWindow
from viewlistingsgui import Ui_listingsViewDialog

#------------------------------------------------------------------------------------------
#These are the strings used to connect to the online database, 
#    we simply inject our vars into them.
#
#Need to be able to configure this easily, as they change how the site works once and a while

SERVER = "www.whitepages.com"
ADDRWEBADDR = '/search/ReverseAddress?street=%s&city_zip=%s&state_id=%s&localtime=survey&redir=2&lower=%s'   

#------------------------------------------------------------------------------------------
#This function is given a string, if it's empty, it returns 'NA'
#    so that the exported record shows at least something.
def ClearBlank(data):
    if data == '' or str(data) == 'None': #In case it's a None type object.
        return 'NA'
    else:
        return data
    
#------------------------------------------------------------------------------------------
#Make sure there is a proper file extension.
def CheckExtension(file):
        x = file.split('.')
        if x[len(x)-1] != 'pdf':
            return file + '.pdf'
        else:
            return file
          
#------------------------------------------------------------------------------------------
#This little class connects to QListWidgetItem allowing us to use a custom
#    system for keep track of the lists contents regardless of display text.
class listItem_c(QListWidgetItem):
    def __init__(self, listing):
        QListWidgetItem.__init__(self, ClearBlank(listing['street_address']) + ' - ' + 
                                 ClearBlank(listing['phone_num']))
        self.listing = listing

#-----------------------------------------------------------------------------------------  
#Custom sorting anyone? ...
#10/10/08 - 
#    This just got a lot more complicated. First sort on street name, then on address.
def SortListingsByHouseNumber(listing1, listing2):
    for i in range(len(listing1['street'])):
        if ord(listing1['street'][i]) != ord(listing2['street'][i]):
            return ord(listing1['street'][i]) - ord(listing2['street'][i])
    return int(listing1['house_num']) - int(listing2['house_num'])

#------------------------------------------------------------------------------------------
# These are the RE patterns and such needed to parse the data retreived from the website.
#   To keep the code clean they are kept here with the 'results' class, which _should_
#   be the only place they are used.
numOfResults_rep = re.compile('.*?<strong>(\d+).*?', re.DOTALL)
listing_rep = re.compile("""
    .*?                                            #1 by 1 search to startpoint
    \"results_multiple_listing_container_\d+?\"    #Startpoint
    .*?class=\"type\">(.*?)</p>                    #Group 0: Listing type
    .*?\"postal-code\">(.*?)</span>                #Group 1: Zip
    .*?(?:class=\"phone\stel\">(.+?)</p>|          #Group 2: Telephone number
    phone\snumber\s(unavailable))                  # ... or not
    .*?\"resultslink_(\d+?)\",\s+?                 #Group 4: Listing number
    \'FIRST\',\s+?\"(.*?)\",\s+?                   #Group 5:
    \'LAST\',\s+?\"(.*?)\",\s+?                    #Group 6:
    \'ADDRESS\',\s+?\"(.*?)\",\s+?                 #Group 7:
    \'CITY\',\s+?\"(.*?)\",\s+?                    #Group 8:
    \'STATE\',\s+?\"(.*?)\",\s+?                   #Group 9:
    .*?</li>.*?                                    #The end!
    """, re.DOTALL | re.VERBOSE)
houseNumber_rep = re.compile('(\d+)\D*?')
streetName_rep = re.compile('\d+\s(.*)', re.DOTALL)

#------------------------------------------------------------------------------------------
#A simple class to help keep track of how much progress has been/needs to be done.
class progress_c():
    def __init__(self):
        self.todo = 0
        self.done = 0
        
#------------------------------------------------------------------------------------------
#This class is used to pass the export configuration to the exporting plugin.
class ExportConfig:
    def __init__(self, labelTextFont, labelTextSize,
                 mainTextFont, mainTextSize, insideBorder, outsideBorder,
                 header):
        self.labelTextFont = labelTextFont
        self.labelTextSize = labelTextSize
        self.mainTextFont = mainTextFont
        self.mainTextSize = mainTextSize
        self.labelBorder = insideBorder
        self.mainBorder = outsideBorder
        self.header = header

#------------------------------------------------------------------------------------------
############## START OF results_c ##############
#------------------------------------------------------------------------------------------

class results_c(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self.listings = []
        self.selectedListings = []
        self.matches = 0
        self.progress = progress_c()

#------------------------------------------------------------------------------------------

    def GetListings(self, street='', city_zip='', state=''):
        self.__ClearProgress__()
        self.__ClearFoundListings__()
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Getting listings...")
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Connecting...")
        if len(street) == 0 or len (city_zip) == 0 or len (state) == 0:
            self.emit(QtCore.SIGNAL("error(QString)"),
                "You forgot to fill in all the information.")
            self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Error: No info given to lookup.\n")
            self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished.")
            return
        data = self.__GetData__(street, city_zip, state)
        if data == '':
            self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Error connecting and getting data.\n")
            self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished. Error occurred.")
            return
        #Get the about of listings availible.
        currentMatches = self.__ParseMatches__(data)
        self.__RefreshWindow__()
        if currentMatches == 0:
            self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished. No listings found.")
            return
        #Calculate how much needs to be done.
        self.__CalculateWork__(currentMatches)
        #Step progress once because we allready connected.
        self.__StepProgress__()
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Found %i listings." % 
                   currentMatches)
        self.matches = self.matches + currentMatches
        if self.matches >= 1:
            matchedListings = self.__ParseListings__(data)
            while len(self.listings) < self.matches:
                self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Getting listings...")
                self.emit(QtCore.SIGNAL("appendDebugText(QString)"),
                          "Getting more data. At: %s of %s" % 
                          (str(len(self.listings)), self.matches))
                self.__RefreshWindow__()
                data = self.__GetData__(street, city_zip, state, start_num=str(matchedListings + 1))
                if data == '':
                    self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Error connecting and getting data.\n")
                    self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished. Error occurred.")
                    return
                #Because we connected to get data and I don't want to call this from __GetData__ like I should
                self.__StepProgress__()
                matchedListings = matchedListings + self.__ParseListings__(data)
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished.")

#------------------------------------------------------------------------------------------
#This removes all the class's duplicate listings based on the phone number.
    def RemoveDupes(self):
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Removing duplicates...")
        nums = {}
        dupes = []
        for listing in self.listings:
            if nums.has_key(listing['phone_num']):
                dupes.append(listing)
            else:
                nums[listing['phone_num']] = 1
        for dupe in dupes:
            self.listings.remove(dupe)
            self.matches = self.matches - 1
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Removed %i duplicate listings." % 
                   len(dupes))
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished.")

#------------------------------------------------------------------------------------------
#This removes all the class's business listings.
    def RemoveWork(self):
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Removing businessess...")
        businesses = []
        for listing in self.listings:
            if listing['type'] == 'work':
                businesses.append(listing)
        for business in businesses:
            self.listings.remove(business)
            self.matches = self.matches - 1
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Removed %i business listings." % 
                   len(businesses))
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished.")
        
#------------------------------------------------------------------------------------------
#Little function that just starts the class over with new lists.
    def ClearAll(self):
        self.matches = 0
        self.listings = []
        self.selectedListings = []
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Cleared lists.")
        
#------------------------------------------------------------------------------------------
#Little function hat just starts the class over with new lists.
    def ClearSelected(self):
        self.selectedListings = []
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Cleared selected listings.")

#------------------------------------------------------------------------------------------
#Copy all listings to the selectedListings list, while avoiding dupes, then sort.
    def SelectAll(self):
        for listing in self.listings:
            match = False
            for selectedListing in self.selectedListings:
                if listing['phone_num'] == selectedListing['phone_num']:
                    match = True
                    break
            if match == True:
                continue
            self.selectedListings.append(listing)
        self.selectedListings.sort(SortListingsByHouseNumber)
        self.emit(QtCore.SIGNAL("updateLists"))
        
#------------------------------------------------------------------------------------------          
#This is the function that connects to the web site and grabs the data.
#It is given 3 strings, plus 1 optional: 
#    Street address (or a street name. not required, can be blank),    
#    City name, or zip code (cannot be blank),
#    State initials (cannot be blank),
#    Listing number to start from. For multiple pages of listings.
#        This tell the site to start the page from this listing.
#
#This function returns the HTML document given by the web site.
    def __GetData__(self, street, city_zip, state_id, start_num='1'):
        try:
            conn = httplib.HTTPConnection(SERVER)
            conn.request('GET', ADDRWEBADDR % (street, city_zip, state_id, start_num))
            response = conn.getresponse()
            data = response.read()
        except:
            self.emit(QtCore.SIGNAL("error(QString)"),
                'You don\'t appear to have an active internet connection\n' + 
                'Please check your network settings and try again')
            return ''
        conn.close()
        return data

#------------------------------------------------------------------------------------------
#This function is given the HTML doc retrieved and 
#    uses the RE patterns to extract the data.
#It appends the listings to the list, self.listings, as dictionaries 
#    that contain each part of the listing.
    def __ParseListings__(self, data):
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Parsing listings...")
        self.__RefreshWindow__()
        lock = thread.allocate_lock()
        try:
            matches = 0
            iterator = listing_rep.finditer(data)
            self.__RefreshWindow__()
            for match in iterator:
                if match == None: #Needed?
                    break
                listing = {} #Start with a blank dict each iteration
                listing['type'] = ClearBlank(match.group(1))
                listing['zip'] = ClearBlank(match.group(2))
                listing['phone_num'] = ClearBlank(match.group(3))
                listing['listing_num'] = ClearBlank(match.group(4))
                listing['first_name'] = ClearBlank(match.group(6))
                if listing['type'] != 'work':
                    listing['last_name'] = ClearBlank(match.group(7))
                    listing['name'] = listing['last_name'] + ', ' + listing['first_name']
                else:
                    listing['name'] = listing['first_name']
                listing['street_address'] = ClearBlank(match.group(8))
                listing['street'] = streetName_rep.match(listing['street_address']).group(1)
                listing['house_num'] = houseNumber_rep.match(listing['street_address']).group(1)
                listing['city'] = ClearBlank(match.group(9))
                listing['state'] = ClearBlank(match.group(10))
                listing['full_address'] = listing['street_address'] + '\n' + \
                    listing['city'] + ' ' + listing['state'] + ', ' + listing['zip']
                with lock:
                    self.listings.append(listing)
                self.__StepProgress__()
                self.__RefreshWindow__()
                matches = matches + 1
            return matches
        except:
            self.__ParseError__(data)
            
#------------------------------------------------------------------------------------------
#Simple function that takes the retrieved HTML doc and
#    grabs the total number of results.
    def __ParseMatches__(self, data):
        try:
            m = numOfResults_rep.match(data)
            n = m.groups()[0]
        except:
            self.__ParseError__(data)
        return int(n)
    
#------------------------------------------------------------------------------------------
#This should be called whenever there is a problem with 
#    the functions in this class.
    def __ParseError__(self, data):
        self.emit(QtCore.SIGNAL("error(QString)"), 
            'Retrieved data is formed in an unknown manner, or there was an error parsing.\n' +
            'Check for an update for this program.\n' + 
            'Debug info dumped to: \'teletory.debug\'')
        debugFile = open('teletory.debug', 'w')
        debugFile.write(data)
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Error occurred:")
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), str(sys.exc_info()[0]))
        self.emit(QtCore.SIGNAL("changeStatus(QString)"), "Finished. Error occurred.")
        sys.exit()
        
#------------------------------------------------------------------------------------------
#Calculate how much needs to be done.
    def __CalculateWork__(self, matches):
        timesToConnect = matches / 10 #Integer division, rounded down
        #Add a time if there is a remainder.
        if matches % 10 > 0:
            timesToConnect = timesToConnect + 1
        self.progress.todo = matches + timesToConnect
            
#------------------------------------------------------------------------------------------
#Move a step forward in progress and display it.
    def __StepProgress__(self):
        self.progress.done = self.progress.done + 1
        #If this next one is confusing I understand...
        #Calculate the percent completed and display.
        self.emit(QtCore.SIGNAL("progress(int)"), 
                  int((float(self.progress.done)/self.progress.todo)*100))
        
#------------------------------------------------------------------------------------------
#Clear all progress.
    def __ClearProgress__(self):
        self.progress.done = 0
        self.progress.todo = 0
        self.emit(QtCore.SIGNAL("progress(int)"), 0)
        
#------------------------------------------------------------------------------------------
#Empty the found listing list.
    def __ClearFoundListings__(self):
        self.matches = 0
        self.listings = []
        self.emit(QtCore.SIGNAL("updateLists"))
        self.emit(QtCore.SIGNAL("appendDebugText(QString)"), "Cleared lists.")
    
#------------------------------------------------------------------------------------------
#This function sends a signal to handle all events then pauses a bit to let it work.
#    This prevents highly extensive functions from bogging it down.
    def __RefreshWindow__(self, secs=0.5):
        self.emit(QtCore.SIGNAL("refreshWindow"))
        time.sleep(secs) #Give it a little time to refresh. Configure this?

#------------------------------------------------------------------------------------------
############## END OF results_c ##############
#------------------------------------------------------------------------------------------

#------------------------------------------------------------------------------------------
############## mainWin_c ##############
#    This is the class we use to inherit the view listings dialog.
#------------------------------------------------------------------------------------------

class viewListings_c(QDialog, Ui_listingsViewDialog):
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.setupUi(self)

#------------------------------------------------------------------------------------------
############## mainWin_c ##############
#    This is the class that inherits the class from the designer output.
#    We use this to setup the signals and slots for program opperation.
#        We do this per notebook tab, via seperate functions. To make it cleaner,
#------------------------------------------------------------------------------------------

class mainWin_c(QMainWindow, Ui_MainWindow):
    def __init__(self):
        QDialog.__init__(self)
        
        # Setup a blank results class to use for our activity.
        self.results = results_c()

        # Set up the user interface from Designer.
        self.setupUi(self)
        self.__SetupDebugTab__()
        self.__SetupFindTab__()
        self.__SetupExportTab__()
        self.__SetupMenuBar__()
        self.__SetupMiscSlots__()

#------------------------------------------------------------------------------------------
    def __SetupDebugTab__(self):
        self.connect(self.results, QtCore.SIGNAL('appendDebugText(QString)'),
                self.debugConsole, QtCore.SLOT('appendPlainText(QString)'))
        self.connect(self.clearDebugConsoleButton, QtCore.SIGNAL("clicked()"),
                     self.debugConsole, QtCore.SLOT("clear()"))

#------------------------------------------------------------------------------------------
    def __SetupFindTab__(self):
        self.connect(self.retriveListingsButton, QtCore.SIGNAL("clicked()"),
                     self.ProcessFormData)
        self.connect(self.removeDupesButton, QtCore.SIGNAL("clicked()"),
                     self.results.RemoveDupes)
        self.connect(self.removeWorkButton, QtCore.SIGNAL("clicked()"),
                     self.results.RemoveWork)
        #This will be called whenever we need to refresh the lists
        self.connect(self.results, QtCore.SIGNAL("updateLists"),
                     self.RefreshListingLists)
        self.connect(self.addButton, QtCore.SIGNAL("clicked()"),
                     self.SelectListings)
        self.connect(self.clearAllButton, QtCore.SIGNAL("clicked()"),
                     self.results.ClearAll)
        self.connect(self.removeAllButton, QtCore.SIGNAL("clicked()"),
                     self.results.ClearSelected)
        self.connect(self.removeButton, QtCore.SIGNAL("clicked()"),
                     self.RemoveSelected)
        self.connect(self.addAllButton, QtCore.SIGNAL("clicked()"),
                     self.results.SelectAll)
        self.connect(self.clearFormButton, QtCore.SIGNAL("clicked()"),
                     self.ClearForm)
        self.connect(self.deselectAllButton, QtCore.SIGNAL("clicked()"),
                     self.DeselectListings)
        self.connect(self.viewSelectedFoundButton, QtCore.SIGNAL("clicked()"),
                     lambda : self.ViewListings(self.foundListingsList.selectedItems()))
        self.connect(self.viewAllFoundButton, QtCore.SIGNAL("clicked()"),
                     lambda : self.ViewListings(self.foundListingsList.findItems('.*', 
                            QtCore.Qt.MatchFlag(QtCore.Qt.MatchRegExp))))
        self.connect(self.viewSelectedSavedButton, QtCore.SIGNAL("clicked()"),
                     lambda : self.ViewListings(self.selectedListingsList.selectedItems()))
        self.connect(self.viewAllSavedButton, QtCore.SIGNAL("clicked()"),
                     lambda : self.ViewListings(self.selectedListingsList.findItems('.*',
                            QtCore.Qt.MatchFlag(QtCore.Qt.MatchRegExp))))

#------------------------------------------------------------------------------------------
    def __SetupExportTab__(self):
        self.connect(self.exportSaveButton, QtCore.SIGNAL("clicked()"),
                     self.Export)
        self.connect(self.exportFileBrowseButton, QtCore.SIGNAL("clicked()"),
                     self.SelectFile)

#------------------------------------------------------------------------------------------
    def __SetupMenuBar__(self):
        self.connect(self.actionExit, QtCore.SIGNAL("triggered()"), qApp, QtCore.SLOT("quit()"))
        self.connect(self.results, QtCore.SIGNAL('changeStatus(QString)'), self.statusbar,
                     QtCore.SLOT('showMessage(QString)'))
        #Show a message in statusbar to start off with.
        self.statusbar.showMessage("Finished loading.")
    
#------------------------------------------------------------------------------------------
    def __SetupMiscSlots__(self):
        self.connect(self.results, QtCore.SIGNAL("error(QString)"), self.ErrorMessage)
        self.connect(self.results, QtCore.SIGNAL("progress(int)"), self.progressBar,
                    QtCore.SLOT("setValue(int)"))
        self.connect(self.results, QtCore.SIGNAL("refreshWindow"), QApplication.instance().processEvents,
                     QtCore.Qt.DirectConnection)
        
#------------------------------------------------------------------------------------------
    def ClearForm(self):
        self.stateEdit.clear()
        self.cityZipEdit.clear()
        self.streetEdit.clear()
        
#------------------------------------------------------------------------------------------
#This handy little function grabs the text from the find listings
#    form and returns them as python strings cleaned and formatted.
#    Then, this function takes the liberty of calling 
#    results_c.GetListings() using those strings.
    def ProcessFormData(self):
        street = str(self.streetEdit.text()).replace(' ', '+')
        city = str(self.cityZipEdit.text()).replace(' ', '+')
        state = str(self.stateEdit.text())
        thread.start_new_thread(self.results.GetListings, (street, city, state))
        
#------------------------------------------------------------------------------------------
    def RefreshListingLists(self):
        self.debugConsole.appendPlainText("Refreshing listings lists...")
        self.statusbar.showMessage("Refreshing listing lists...")
        self.foundListingsList.clear()
        for listing in self.results.listings:
            self.foundListingsList.addItem(listItem_c(listing))
            QApplication.instance().processEvents()
        self.foundListingsLcd.display(len(self.results.listings))
        self.selectedListingsList.clear()
        for listing in self.results.selectedListings:
            self.selectedListingsList.addItem(listItem_c(listing))
            QApplication.instance().processEvents()
        self.selectedListingsLcd.display(len(self.results.selectedListings))
        self.statusbar.showMessage("Finished.")
        
#------------------------------------------------------------------------------------------           
#Not to be confuse as the counterpart to DeselectListings().
#    This guy adds listings to the saved listing list.
    def SelectListings(self):
        match = False
        for item in self.foundListingsList.selectedItems():
            match = False
            for savedListing in self.results.selectedListings:
                if item.listing['phone_num'] == savedListing['phone_num']:
                    match = True
                    break    
            if match == True:
                continue
            self.results.selectedListings.append(item.listing)
        #Put them in ascending order.
        self.results.selectedListings.sort(SortListingsByHouseNumber)
        self.RefreshListingLists()
        
#------------------------------------------------------------------------------------------           
#Go through each selected listing and make it not selected.
    def DeselectListings(self):
        for item in self.foundListingsList.selectedItems():
            self.foundListingsList.setItemSelected(item, False)
        for item in self.selectedListingsList.selectedItems():
            self.selectedListingsList.setItemSelected(item, False)
        
#------------------------------------------------------------------------------------------           
#This guy is given a list of listitems, parses the listings contained in them, and opens
#    a viewListingsDialog to display them in plain text.
    def ViewListings(self, listitems):
        self.debugConsole.appendPlainText("Viewing %i listings." % len(listitems))
        dialog = viewListings_c(self)
        for item in listitems:
            dialog.viewListingsTextEdit.appendPlainText(item.listing['name'])
            dialog.viewListingsTextEdit.appendPlainText(item.listing['full_address'])
            dialog.viewListingsTextEdit.appendPlainText(item.listing['phone_num'] + "\n")
        dialog.show()
        
#------------------------------------------------------------------------------------------ 
    def RemoveSelected(self):
        for item in self.selectedListingsList.selectedItems():
            self.selectedListingsList.takeItem(self.selectedListingsList.row(item))
            self.results.selectedListings.remove(item.listing)
        #Refresh the LCD here, because we don't need to call RefreshListingsList.
        self.selectedListingsLcd.display(len(self.results.selectedListings))
        
#------------------------------------------------------------------------------------------
#Let the user pick the file they want to export to, then put it in the line edit.
    def SelectFile(self):
        fileName = QFileDialog.getSaveFileName(self, 'Export to PDF', '',
                    'Portable Document Format (*.pdf)')
        self.exportFileEdit.setText(CheckExtension(fileName))

#------------------------------------------------------------------------------------------
#Get the configuration for the exported document and do it!
    def Export(self):
        mainTextFont = self.mainTextFontBox.currentFont()
        labelTextFont = self.labelTextFontBox.currentFont()
        config = ExportConfig((labelTextFont.key(), labelTextFont.family()),
                  self.labelTextSizeBox.value(),
                  (mainTextFont.key(), mainTextFont.family()),
                  self.mainTextSizeBox.value(),
                  self.labelBorderSlider.value(),
                  self.mainBorderSlider.value(),
                  str(self.headerTextEdit.text()))
        ExportPDF(self.results.selectedListings, 
                  CheckExtension(self.exportFileEdit.text()), config)

#------------------------------------------------------------------------------------------
    def ErrorMessage(self, message):
        QMessageBox.critical(None, 'Error!', message,
            QMessageBox.Ok)        

#------------------------------------------------------------------------------------------
############## END OF mainWin_c ##############
#------------------------------------------------------------------------------------------

def main():
    app = QApplication(sys.argv)
    MainWin = mainWin_c()
    MainWin.show()
    sys.exit(app.exec_())

#------------------------------------------------------------------------------------------

if __name__ == '__main__':
    main()